/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD027-1B.h"
#include "Debug.h"
#include "GUI_Paint.h"

const unsigned char lut_vcom_dc[] = {
    0x00, 0x00, 
    0x00, 0x0F, 0x0F, 0x00, 0x00, 0x05,      
    0x00, 0x32, 0x32, 0x00, 0x00, 0x02,      
    0x00, 0x0F, 0x0F, 0x00, 0x00, 0x05,      
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,      
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,      
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,      
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

//R21H
const unsigned char lut_ww[] = {
    0x50, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x60, 0x32, 0x32, 0x00, 0x00, 0x02,
    0xA0, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

//R22H    r
const unsigned char lut_bw[] =
{
    0x50, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x60, 0x32, 0x32, 0x00, 0x00, 0x02,
    0xA0, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

//R24H    b
const unsigned char lut_bb[] =
{
    0xA0, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x60, 0x32, 0x32, 0x00, 0x00, 0x02,
    0x50, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

//R23H    w
const unsigned char lut_wb[] =
{
    0xA0, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x60, 0x32, 0x32, 0x00, 0x00, 0x02,
    0x50, 0x0F, 0x0F, 0x00, 0x00, 0x05,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD2IN7::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD2IN7::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD2IN7::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD2IN7::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == 0) {      //LOW: idle, HIGH: busy
        Dev->Dev_Delay_ms(100);
    }
}
/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD2IN7::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_REFRESH);
    EPD_WaitUntilIdle();
}

/******************************************************************************
function :	set the look-up tables
parameter:
******************************************************************************/
void EPD2IN7::EPD_SetLut(void) {
    unsigned int count;     
    EPD_SendCommand(LUT_FOR_VCOM);                            //vcom
    for(count = 0; count < 44; count++) {
        EPD_SendData(lut_vcom_dc[count]);
    }
    
    EPD_SendCommand(LUT_WHITE_TO_WHITE);                      //ww --
    for(count = 0; count < 42; count++) {
        EPD_SendData(lut_ww[count]);
    }   
    
    EPD_SendCommand(LUT_BLACK_TO_WHITE);                      //bw r
    for(count = 0; count < 42; count++) {
        EPD_SendData(lut_bw[count]);
    } 

    EPD_SendCommand(LUT_WHITE_TO_BLACK);                      //wb w
    for(count = 0; count < 42; count++) {
        EPD_SendData(lut_bb[count]);
    } 

    EPD_SendCommand(LUT_BLACK_TO_BLACK);                      //bb b
    for(count = 0; count < 42; count++) {
        EPD_SendData(lut_wb[count]);
    } 
}

/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD2IN7::EPD_Init(void)
{
    Dev->System_Init();

    EPD_Reset();;
    EPD_SendCommand(POWER_SETTING);
    EPD_SendData(0x03);                  // VDS_EN, VDG_EN
    EPD_SendData(0x00);                  // VCOM_HV, VGHL_LV[1], VGHL_LV[0]
    EPD_SendData(0x2b);                  // VDH
    EPD_SendData(0x2b);                  // VDL
    EPD_SendData(0x09);                  // VDHR
    
    EPD_SendCommand(BOOSTER_SOFT_START);
    EPD_SendData(0x07);
    EPD_SendData(0x07);
    EPD_SendData(0x17);
    // Power optimization
    EPD_SendCommand(0xF8);
    EPD_SendData(0x60);
    EPD_SendData(0xA5);
    // Power optimization
    EPD_SendCommand(0xF8);
    EPD_SendData(0x73);
    EPD_SendData(0x23);
    // Power optimization
    EPD_SendCommand(0xF8);
    EPD_SendData(0x7C);
    EPD_SendData(0x00);

    EPD_SendCommand(PARTIAL_DISPLAY_REFRESH);
    EPD_SendData(0x00);
    EPD_SendCommand(POWER_ON);
    EPD_WaitUntilIdle();

    EPD_SendCommand(PANEL_SETTING);
    EPD_SendData(0x1F);        //KW-BF   KWR-AF    BWROTP 0f
    EPD_SendCommand(PLL_CONTROL);
    EPD_SendData(0x3A);       //3A 100HZ   29 150Hz 39 200HZ    31 171HZ
  
    EPD_SendCommand(TCON_RESOLUTION);
    EPD_SendData(0x00);
    EPD_SendData(0xB0);
    EPD_SendData(0x01);
    EPD_SendData(0x08);
    
    
    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD2IN7::EPD_Clear(void) 
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);           
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD2IN7::EPD_Display(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);           
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD2IN7::EPD_Sleep(void) 
{
    EPD_SendCommand(DEEP_SLEEP);
    EPD_SendData(0xa5);
}
